#[repr(C)]pub enum DmapField {
Vector(DmapVec),
Scalar(DmapScalar),
}Expand description
A generic field of a DMAP record.
This is the type that is stored in a DMAP record, representing either a scalar or vector field.
Variants
Vector(DmapVec)
Scalar(DmapScalar)
Implementations